local ADDON_NAME, Core = ...

--<
local format, gsub, upper, match = string.format, string.gsub, string.upper, string.match
local CreateFrame = CreateFrame
-->

Core.VERSION = "1.3.6"
Core.TAB_HEIGHT = 34
Core.LINE_HEIGHT = 14
Core.utils = {}

-------------------------------------------------------------------
-- Utility & Helpers

function Core.utils.capitalize (str)
	return str:lower():gsub("^%l", upper)
end


function Core.utils.any (key, ...)
	local isAny = false
	
	for i=1, select("#", ...) do
		if key == select(i, ...) then
			isAny = true
			break
		end
	end
	
	return isAny
end


function Core.utils.GetColouredPlayerLink (name, englishClass, rpalias)
	local classColour = RAID_CLASS_COLORS[englishClass]
	return classColour:WrapTextInColorCode(GetPlayerLink(name, rpalias or Ambiguate(name, "none")))
end


function Core.utils.GetRGB (ctype)
	local colour =  ChatTypeInfo[ctype]
	return colour.r, colour.g, colour.b
end


local function showGameTooltip (frame)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	local title, body = string.match(frame.tip, "(.*)%\n#(.*)")
	
	if title and body then	
		GameTooltip_SetTitle(GameTooltip, title)
		GameTooltip:AddLine(body)
	else
		GameTooltip:SetText(frame.tip)
	end
	
	GameTooltip:Show()
end


function Core.utils.RegisterForTooltip (frame)
	frame:SetScript("OnEnter", showGameTooltip)
	frame:SetScript("OnLeave", GameTooltip_Hide)
end


function Core.utils.GetEpochFromString (dt)
	local y, m, d = string.match(dt, "(%d+)-(%d+)-(%d+)")
	return time({year=y, month=m, day=d})
end


function Core.utils.getFullName (name, realm)
	if name:match("-") then return name end	
	return format("%s-%s", name, (realm ~= "" and realm) or Core.myRealmName)
end


function Core.utils.SMPrint (msg)
	print( format("%s(SM) %s|r", RGBToColorCode(0.45, 0.8, 0.7), msg) )
end


function Core.utils.AddOrRemoveEvents (frame, rm, ...)
	for i=1, select("#", ...) do
		if rm then
			frame:UnregisterEvent(select(i, ...))
		else
			frame:RegisterEvent(select(i, ...))
		end
	end
end


function Core.utils.SaveFramePosition (frame, layout)
	local layout = Core.db[layout]
	
	local point, _, rel, x, y = frame:GetPoint()
	layout.point = point; layout.rel = rel; layout.x = x; layout.y = y
end


function Core.utils.ParseURL (text)
	if text:match("[^<]https?://") then
		text = gsub(text, "https?://[%w%p]+", "|H%1|h|cFF3366BB<%1>|r|h")
	else
		text = gsub(text, "[%w%-%.]+%.[%w%-]+%/[%w%p]+", "|Hhttp://%1|h|cFF3366BB<http://%1>|r|h")
	end
	
	return text
end


function Core.utils.AddError (err)
	if not Core.debugging then Core.debugging = {} end
	Core.debugging[#Core.debugging+1] = err
end
-------------------------------------------------------------------


-------------------------------------------------------------------
local UI = CreateFrame("Frame", ADDON_NAME.."_MainFrame", UIParent, "BackdropTemplate")
-------------------------------------------------------------------


function UI:GetMaxTabNum ()
	return math.floor(self.leftFrame:GetHeight() / Core.TAB_HEIGHT)
end


function UI:PseudoScrollUpdate ()
	local TOP, BOTTOM = self.tabWindow:GetTop(), self.tabWindow:GetBottom()
	local offset = Core.TAB_HEIGHT / 2
	
	for i=1, #Core.orderedTabs do
		local tab = Core.orderedTabs[i]:GetBottom()
		
		if ((tab + offset) < TOP) and (tab > BOTTOM) then
			Core.orderedTabs[i]:Show()

		else
			Core.orderedTabs[i]:Hide()
		end
	end

end


local function tabWindow_UpdateOffset (f, delta)
	local pos = math.floor(select(5, f.manager:GetPoint()))
	local H = Core.TAB_HEIGHT

	if (delta == 1) and not (pos - H < 0) then
		f.manager:SetPoint("TOP", 0, (pos - H))
	
	elseif (delta == -1) and not (pos + H > (#Core.orderedTabs - UI:GetMaxTabNum()) * H) then
		f.manager:SetPoint("TOP", 0, pos + H)

	end
	
	UI:PseudoScrollUpdate()
	
end


function UI.OnMouseWheelEvent (this, delta)
	local vs = math.ceil(this:GetVerticalScroll())

	if (vs == 0) and (delta ~= -1) then
		return
	end
	
	UI.editBoxSlider:SetValue( (delta == -1 and vs + Core.LINE_HEIGHT) or (delta == 1 and vs - Core.LINE_HEIGHT) )
end


function UI:OnCursorEvent (y)
	local vs = math.ceil(self.editBoxScroller:GetVerticalScroll())

	if (vs + y - Core.LINE_HEIGHT + self.editBoxViewport) < 0 then
		self:SetSliderValues()
		self.editBoxSlider:SetValue(math.ceil((y * -1) - (self.editBoxViewport - Core.LINE_HEIGHT)))

	elseif (vs + y) > 0 then
		self.editBoxSlider:SetValue(y*-1)
	end
end


function UI:SetSliderValues ()
	local ebHeight = self.editBox:GetHeight()
	local viewport = self.editBoxViewport
	
	self.editBoxSlider:SetShown(ebHeight > viewport)
	self.editBoxSlider:SetMinMaxValues(0, ebHeight >= viewport and (ebHeight-viewport) or 0)
end


function UI:Toggle ()
	self:SetShown(not self:IsShown())
end


function UI:OpenChat ()
	if not self:IsShown() then
		self:Show()	
	end
	
	self.editBox:SetFocus()
end


function UI:StopMoving ()
	self:StopMovingOrSizing()
	Core.utils.SaveFramePosition(self, "UILayout")
end


function UI:UpdateHeader (session)
	local r, g, b

	if session == nil then
		r, g, b = 0, 0, 0
		self.header.title:SetText("")
		self.body:SetBackdropBorderColor(1, 1, 1)
		self.lowerFrame:SetBackdropBorderColor(1, 1, 1)
	else
		r, g, b = session:GetRGB()
		self.header.title:SetText("@ ".. session.title)
		self.body:SetBackdropBorderColor(r, g, b)
		self.lowerFrame:SetBackdropBorderColor(r, g, b)
	end

	self.header.title:SetTextColor(r, g, b)
	self.lowerFrame.BG:SetGradient("HORIZONTAL", r*0.1,g*0.1,b*0.1, 0,0,0)
end


local function inputBox_OnFocusLost (self)
	if Core.suggestionFrame:IsMouseOver() then 
		return
	else
		Core.suggestionFrame:Detach()
	end
end


local function statusIcon_OnEvent (self)
	local status = (UnitIsAFK("player") and "Away") or (UnitIsDND("player") and "DND") or "Online"	
	self.tip = "Appearing ".. status
	self:SetNormalTexture("Interface/FriendsFrame/StatusIcon-".. status)
end


local function editBox_CharacterCounter (self)
	local num = self:GetNumLetters()
	
	if num == 0 then
		return
	end
	
	paras = math.floor(num / 255)
	currentPara = num % 255
	self.counter.number:SetText(currentPara.."/"..paras)
	self.counter:Show()
end


local function editBox_OnTextChanged (self, byUser)
	if (not byUser) or (self:GetText() == "") then
		return
	end

	local S = Core.Sessions.selected

	if time() >= S.typeInform_lastSent then
		ChatThrottleLib:SendAddonMessage("NORMAL", "SessionsMSGR", "TI01", "WHISPER", S.senderID)
		S.typeInform_lastSent = time()+7
	end
end


local function editBox_OnEditFocusGained (self)
	ACTIVE_CHAT_EDIT_BOX = self
	self:SetWidth(UI.lowerFrame:GetWidth()-27)
	UI.lowerFrame.BG:Show()
	UI.lowerFrame:SetBackdropBorderColor(Core.Sessions:GetSelectedRGB())
	self:AddOrRemoveOnTextHandler()
end


local function editBox_OnEditFocusLost (self)
	ACTIVE_CHAT_EDIT_BOX = nil
	Core.G_EditBox:SetAttribute("chatType", Core.G_ChatType)
	UI.lowerFrame.BG:Hide()
	UI.lowerFrame:SetBackdropBorderColor(1, 1, 1)
end


local function editBox_AddOrRemoveOnTextHandler (self)
	if Core.Sessions.selected and Core.Sessions.selected.isUsingSM then
		self:SetScript("OnTextChanged", editBox_OnTextChanged)
	else
		self:SetScript("OnTextChanged", nil)
	end
end


local function editBox_OnSizeChanged (self, w, h)
	if h <= UI.editBoxViewport then
		UI:SetSliderValues()
	end
end


function UI.OnSizeChanged (self, width, height)
	if height < 230 and self.editBoxViewport ~= 28 then
		self.editBoxViewport = 2 * Core.LINE_HEIGHT
	elseif height > 230 and self.editBoxViewport ~= 42 then
		self.editBoxViewport = 3 * Core.LINE_HEIGHT
	else return
	end
	
	self.lowerFrame:SetHeight(self.editBoxViewport + 8)
end

-------------------------------------------------------------------
-- Initialize 
-------------------------------------------------------------------
do
	local self = UI

	self:SetDontSavePosition(true)
	self:SetClampedToScreen(true)
	self:SetMovable(true)
	self:SetResizable(true)
	self:SetMaxResize(1200, 800)
	self:SetMinResize(450, 150)	
	self:EnableMouse(true)
	self:SetFrameStrata("MEDIUM")
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", UI.StopMoving)
	self:HookScript("OnSizeChanged", UI.OnSizeChanged)
	
	self.BG = self:CreateTexture(nil, "BACKGROUND", nil, 1)
	self.BG:SetPoint("BOTTOMRIGHT", -2, 2)
	self.BG:SetPoint("TOPLEFT", 2, 0)
	
	self.lowerFrame = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.lowerFrame:SetHeight(3*Core.LINE_HEIGHT + 8)
	self.lowerFrame:SetPoint("BOTTOMRIGHT")
	self.lowerFrame:SetPoint("BOTTOMLEFT", 110, 1)
	self.lowerFrame.BG = self.lowerFrame:CreateTexture(nil, "BACKGROUND")
	self.lowerFrame.BG:SetPoint("TOPLEFT", 2, -2)
	self.lowerFrame.BG:SetPoint("BOTTOMRIGHT", -2, 2)
	self.lowerFrame.BG:SetColorTexture(1, 1, 1)
	self.lowerFrame.BG:SetGradient("HORIZONTAL", 0,0,0, 0,0,0)
	self.lowerFrame.BG:Hide()
	self.lowerFrame:SetBackdrop({
		edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
		insets={left=2, right=2, top=2, bottom=2}
	})
	
	---------------
	-- EDITBOX
	---------------
	
	self.editBoxViewport = Core.LINE_HEIGHT*3
	
	self.editBoxScroller = CreateFrame("ScrollFrame", nil, self.lowerFrame)
	self.editBoxScroller:SetPoint("TOPLEFT", 6, -4)
	self.editBoxScroller:SetPoint("BOTTOMRIGHT", -4, 4)
	self.editBoxScroller:EnableMouseWheel(true)
	
	self.editBoxSlider = CreateFrame("Slider", nil, self.editBoxScroller)
	self.editBoxSlider:SetPoint("TOPRIGHT", 3, -2)
	self.editBoxSlider:SetPoint("BOTTOMRIGHT", 3, 2)
	self.editBoxSlider.knob = self:CreateTexture(nil)
	self.editBoxSlider.knob:SetSize(2, 12)
	self.editBoxSlider:SetThumbTexture(self.editBoxSlider.knob)
	self.editBoxSlider.knob:SetColorTexture(.8, .8, .8)
	self.editBoxSlider:SetOrientation("VERTICAL")
	self.editBoxSlider:SetSize(10, self.editBoxViewport)
	self.editBoxSlider:SetMinMaxValues(0, 0)
	self.editBoxSlider:SetValue(0)
	self.editBoxSlider:SetValueStep(Core.LINE_HEIGHT)
	self.editBoxSlider:SetScript("OnValueChanged", function(this, v) self.editBoxScroller:SetVerticalScroll(v) end)
	self.editBoxScroller:SetScript("OnMouseWheel", UI.OnMouseWheelEvent)
	
	self.editBox = CreateFrame("EditBox", nil, self.editBoxScroller)
	self.editBox:SetHeight(self.editBoxViewport)
	self.editBox:SetPoint("TOPLEFT")
	self.editBox:SetHitRectInsets(0, 0, 0, -(self.editBoxViewport)*2)
	self.editBox:SetAutoFocus(false)
	self.editBox:SetMultiLine(true)
	self.editBox:SetFontObject("ChatFontNormal")
	self.editBox:SetScript("OnEscapePressed", self.editBox.ClearFocus)
	self.editBox:SetScript("OnEditFocusGained", editBox_OnEditFocusGained)
	self.editBox:SetScript("OnEditFocusLost", editBox_OnEditFocusLost)
	self.editBox:SetScript("OnCursorChanged", function(this, _, y) self:OnCursorEvent(y) end)
	self.editBox:SetScript("OnEnter", editBox_CharacterCounter)
	self.editBox:SetScript("OnLeave", function(this, m) if m then self.editBox.counter:Hide() end end)
	self.editBox:SetScript("OnSizeChanged", editBox_OnSizeChanged)
	self.editBox.AddOrRemoveOnTextHandler = editBox_AddOrRemoveOnTextHandler

	self.editBox.counter = CreateFrame("Frame", nil, self.lowerFrame)
	self.editBox.counter:SetFrameStrata("DIALOG")
	self.editBox.counter:SetSize(35, 11)
	self.editBox.counter:SetPoint("TOPRIGHT", -5, 5)
	self.editBox.counter.BG = self.editBox.counter:CreateTexture(nil, "OVERLAY")
	self.editBox.counter.BG:SetColorTexture(.7, .25, 0)
	self.editBox.counter.BG:SetAllPoints()	
	self.editBox.counter.number = self.editBox.counter:CreateFontString(nil, "OVERLAY")
	self.editBox.counter.number:SetFontObject("SystemFont_Small")
	self.editBox.counter.number:SetPoint("CENTER")
	self.editBox.counter:Hide()
	
	self.editBoxScroller:SetScrollChild(self.editBox)
	
	---------------
	-- HEADER
	---------------	
	
	self.header = CreateFrame("Frame", nil, self)
	self.header:SetSize(self:GetWidth(), 13)
	self.header:SetPoint("TOPLEFT", 1, 0)
	self.header:SetPoint("TOPRIGHT", -2, 0)
	self.header:SetHitRectInsets(0, 0, -15, 0)
	
	self.header.BG = self:CreateTexture(nil, "BACKGROUND", nil, 2)
	self.header.BG:SetPoint("TOPRIGHT", -2, 0) ; self.header.BG:SetPoint("TOPLEFT", 1, 0)
	self.header.BG:SetHeight(self.header:GetHeight())
	self.header.BG:SetColorTexture(1,1,1)
	self.header.BG:SetGradient("HORIZONTAL", 0.1,0.05,0.19, 0,0,0)
	
	self.header.closeBtn = CreateFrame("Button", nil, self.header)
	self.header.closeBtn:SetSize(16, 16)
	self.header.closeBtn:SetPoint("TOPRIGHT", 1, 2)
	self.header.closeBtn:SetNormalTexture([[Interface\FriendsFrame\ClearBroadcastIcon]])
	self.header.closeBtn:SetScript("OnClick", function(...) self:Hide() end)
	
	self.header.optionsBtn = CreateFrame("Button", nil, self.header)
	self.header.optionsBtn:SetSize(16, 16)
	self.header.optionsBtn:SetPoint("LEFT", 16, 0)
	self.header.optionsBtn:SetNormalTexture([[Interface\Worldmap\Gear_64Grey]])
	self.header.optionsBtn:SetScript("OnClick", function() self.leftFrame.inputBox:ClearFocus(); Core.ShowOptions() end)
	self.header.optionsBtn.tip = "Sessions' Quick Options"
	Core.utils.RegisterForTooltip(self.header.optionsBtn)
	
	self.header.statusIcon = CreateFrame("Button", nil, self.header)
	self.header.statusIcon:SetSize(16, 16)
	self.header.statusIcon:SetPoint("LEFT")
	self.header.statusIcon:RegisterEvent("PLAYER_FLAGS_CHANGED")
	self.header.statusIcon:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.header.statusIcon:SetScript("OnEvent", statusIcon_OnEvent)
	Core.utils.RegisterForTooltip(self.header.statusIcon)
	
	self.header.title = self.header:CreateFontString(nil)
	self.header.title:SetFont([[Fonts\MORPHEUS.ttf]], 20, "OUTLINE")
	self.header.title:SetPoint("LEFT", 115, 0)
	self.header.title:SetJustifyH("LEFT")
	
	self.header.selectiveBtn = CreateFrame("CheckButton", nil, self.header)
	self.header.selectiveBtn:SetPoint("RIGHT", self.header.title, "LEFT")
	self.header.selectiveBtn:SetSize(16, 16)
	self.header.selectiveBtn:SetNormalTexture([[Interface\COMMON\VOICECHAT-SPEAKER]])
	self.header.selectiveBtn:SetCheckedTexture([[Interface\COMMON\VOICECHAT-MUTED]])
	self.header.selectiveBtn:SetScript("OnClick", function(this) Core.Sessions.selected.selectiveModeEnabled = this:GetChecked(); end)
	self.header.selectiveBtn.tip = "Allow All Messages\n#If disabled, only messages from players whom you have\nsessions opened for will be shown; others will be minimized."
	Core.utils.RegisterForTooltip(self.header.selectiveBtn)
	self.header.selectiveBtn:Hide()

	---------------
	-- BODY
	---------------
	
	self.body = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.body:SetPoint("BOTTOMLEFT", self.lowerFrame, "TOPLEFT")
	self.body:SetPoint("BOTTOMRIGHT", self.lowerFrame, "TOPRIGHT")
	self.body:SetPoint("TOP", self.header, "BOTTOM")
	self.body:SetBackdrop({
		edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
	})
	
	
	---------------
	-- LEFT STUFF
	---------------
	
	self.leftFrame = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.leftFrame:SetSize(110, self:GetHeight())
	self.leftFrame:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -13)
	self.leftFrame:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 0, 1)
	self.leftFrame:SetBackdrop({
		edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
	})
	
	local addBox = CreateFrame("EditBox",nil, self.leftFrame, "InputBoxInstructionsTemplate")
	addBox:SetAutoFocus(false)
	addBox:SetPoint("TOPLEFT", 9, -2)
	addBox:SetSize(96, 20)
	addBox.Instructions:SetText("Chat with...")
	addBox:SetScript("OnEditFocusGained", function(this) Core.CreateSuggestionFrame(); Core.suggestionFrame:Attach(this); this:HighlightText(); end)
	addBox:SetScript("OnEditFocusLost", inputBox_OnFocusLost)
	
	self.leftFrame.inputBox = addBox
	
	self.tabWindow = CreateFrame("Frame", nil, self.leftFrame)
	self.tabWindow:SetSize(110, self.leftFrame:GetHeight())
	self.tabWindow:SetPoint("TOP", self.leftFrame.inputBox, "BOTTOM")
	self.tabWindow:SetPoint("BOTTOMRIGHT")
	self.tabWindow:SetScript("OnMouseWheel", tabWindow_UpdateOffset)
	
	self.tabWindow.manager = CreateFrame("Frame", nil, self.tabWindow)
	self.tabWindow.manager:SetSize(1, 1)
	self.tabWindow.manager:SetPoint("TOP")

	---------------
	-- OTHER STUFF
	---------------
	
	self.resizeBtn = CreateFrame("Button", nil, self)
	self.resizeBtn:SetSize(16, 16)
	self.resizeBtn:SetPoint("BOTTOMRIGHT", 10, -10)
	self.resizeBtn:SetNormalTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-UP]])
	self.resizeBtn:SetHighlightTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-Highlight]])
	self.resizeBtn:RegisterForDrag("LeftButton")
	self.resizeBtn:Hide()
	self.resizeBtn:SetScript("OnDragStart", function() self:StartSizing("BOTTOMRIGHT") end)
	self.resizeBtn:SetScript("OnDragStop", function() 
		self:StopMovingOrSizing()
		self.editBox:SetWidth(self.lowerFrame:GetWidth() -27)
		self:PseudoScrollUpdate()
		self.editBoxScroller:SetVerticalScroll(0)
		Core.db.UILayout.width, Core.db.UILayout.height = self:GetSize()
	end)
	
	self.floatingMenu = CreateFrame("Frame", ADDON_NAME.."_FloatingFrame", self, "UIDropDownMenuTemplate")
	
	self:Hide()
end


Core.utils.AddOrRemoveEvents(UI, false,
	"CHAT_MSG_WHISPER","CHAT_MSG_WHISPER_INFORM","CHAT_MSG_BN_WHISPER","CHAT_MSG_BN_WHISPER_INFORM","CHAT_MSG_RESTRICTED",
	"BN_FRIEND_INFO_CHANGED","MODIFIER_STATE_CHANGED","PLAYER_TARGET_CHANGED","GROUP_ROSTER_UPDATE","PORTRAITS_UPDATED",
	"WHO_LIST_UPDATE"
)


Core.UI = UI



---------------------
-- HUB FRAME
---------------------

local Bulletin = CreateFrame("Frame")
--[[
function Bulletin:Init ()
	self:SetClampedToScreen(true)
	
	self.BG = self:CreateTexture(nil)
	self.BG:SetColorTexture(0, 0, 0)
	self.BG:SetPoint("TOPLEFT", 6, -5)
	self.BG:SetPoint("BOTTOMRIGHT", -6, 5)
	self:SetBackdrop({edgeFile="Interface/Tooltips/CHATBUBBLE-BACKDROP", edgeSize=24})
	
	self.title = self:CreateFontString(nil)
	self.title:SetFont("Fonts/FRIZQT__.ttf", 16, "OUTLINE")
	self.title:SetPoint("TOPLEFT", 10, 15)
	
	self.msg = self:CreateFontString(nil)
	self.msg:SetFont("Fonts/ARIALN.ttf", 14)
	self.msg:SetPoint("TOPLEFT", 10, -4)
	self.msg:SetPoint("BOTTOMRIGHT", -3, 5)
	self.msg:SetJustifyH("LEFT")
	
	self.tail = self:CreateTexture(nil)
	self.tail:SetTexture("Interface/Tooltips/CHATBUBBLE-TAIL")
	self.tail:SetSize(32, 32)
	
	self.fadeOut = self:CreateAnimationGroup()
	self.alphaAnim = self.fadeOut:CreateAnimation("Alpha")
	self.alphaAnim:SetStartDelay(2)
	self.alphaAnim:SetDuration(2)
	self.alphaAnim:SetFromAlpha(1)
	self.alphaAnim:SetToAlpha(0)
	self.fadeOut:SetScript("OnFinished", function(...) self:Hide() end)
end
--]]



Core.utils.AddOrRemoveEvents(Bulletin, false, "PLAYER_ENTERING_WORLD", "ADDON_LOADED", "FRIENDLIST_UPDATE", "PLAYER_LEAVING_WORLD")


local function checkIngameFriends ()
	local numFriends = C_FriendList.GetNumOnlineFriends()

	if numFriends >= 1 then
		local friends = {[1] = "In-game friends online:"}
		
		for i=1, numFriends do
			f = C_FriendList.GetFriendInfoByIndex(i)
			englishClass = select(2, GetPlayerInfoByGUID(f.guid))
			
			friends[#friends+1] = "[".. Core.utils.GetColouredPlayerLink(f.name, englishClass).. "]"
		end
		
		Core.utils.SMPrint(table.concat(friends, " "))
	end
end


local elapsed = 0
local function onDrag (self, ms)
	elapsed = elapsed + ms
	if elapsed > 1 then
		elapsed = 0
		self:SetAnchor()
	end
end

local function stopMoving (self)
	self:SetAnchor()
	self:SetScript("OnUpdate", nil)
	self:StopMovingOrSizing()
	Core.utils.SaveFramePosition(self, "HubLayout")
end

local function startMoving (self)
	self:StartMoving()
	self:SetScript("OnUpdate", onDrag)
end

local function hub_Initialize ()
	local self = CreateFrame("Button", nil, UIParent, "SessionsHubTemplate")
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", startMoving)
	self:SetScript("OnDragStop", stopMoving)
	self.notices = {}
	self.cache = {}
	self.tip = "Sessions Messenger Hub\n#Click to open Sessions"
	Core.utils.RegisterForTooltip(self)
	
	self:SetScript("OnClick", function() UI:Toggle() end)
	
	self.SetAnchor = function(self)
		local x = self:GetCenter()
		self.anchorRight = x < (GetScreenWidth() / 2) and true or false
		self:Refresh()
	end
	
	self.Refresh = function (self)
		for i=1, #self.notices do
			local f = self.notices[i]
			f:ClearAllPoints()

			if self.anchorRight then
				f:SetPoint("LEFT", self, "RIGHT", i==1 and 3 or (i-1) * (f:GetWidth() + 17), 2)
			else
				f:SetPoint("RIGHT", self, "LEFT", i==1 and -3 or -((i-1) * (f:GetWidth() + 17)), 2)
			end
		end
	end
	
	self.RemoveNotice = function (self, frame)
		frame:ClearAllPoints()
		frame:Hide()
		
		for i=1, #self.notices do
			if self.notices[i] == frame then
				self.cache[#self.cache+1] = table.remove(self.notices, i)
				break
			end
		end
		
		if #self.notices > 0 then
			self:Refresh()
		end
	end
	
	self.AddNotice = function (self, session, ctype)
		local frame
		if #self.cache > 0 then
			frame = table.remove(self.cache)
		else
			frame = CreateFrame("Frame", nil, self, "SessionsNoticeTemplate")
		end		
		self.notices[#self.notices+1] = frame
		
		local r,g,b = Core.utils.GetRGB(ctype)
		frame.BG:SetGradientAlpha("Horizontal", r,g,b,1, r,g,b,0)
		frame.text:SetText(session.missedMessages)
		
		local name = session:GetName()
		if name:find("^|") then name = Core.utils.capitalize(session.name) end
		
		frame.title:SetText(string.sub(name, 1, 4))
		frame.icon:SetVertexColor(r,g,b)
		frame:SetShown(not UI:IsShown())
		self:Refresh()
		
		return frame
	end

	Core.hub = self

end


local ranOnce = false
Bulletin:SetScript("OnEvent", function (self, e, ...)
	if e == "PLAYER_ENTERING_WORLD" then
		Core.UpdatePlayerInfo()
		C_FriendList.ShowFriends()
		C_Timer.After(4, Core.CreatePublicSessions)

		if ranOnce then
			return
		end
		
		Core.G_EditBox = DEFAULT_CHAT_FRAME.editBox
		ChatTypeInfo["WHISPER"].sticky = 0 ; ChatTypeInfo["BN_WHISPER"].sticky = 0
		
		C_ChatInfo.RegisterAddonMessagePrefix("SessionsMSGR")
		
		Core.RPAddOn = (IsAddOnLoaded("XRP") and "XRP")
			or (IsAddOnLoaded("TotalRP3") and "TRP3")
			or (IsAddOnLoaded("MyRolePlay") and "MRP")	
		
		for i=1, NUM_CHAT_WINDOWS do
			local chatFrame = _G["ChatFrame".. tostring(i)]

			if GetChatWindowInfo(i) ~= "" then
				Core.utils.AddOrRemoveEvents(chatFrame, true, 
					"CHAT_MSG_WHISPER", "CHAT_MSG_WHISPER_INFORM",
					"CHAT_MSG_BN_WHISPER", "CHAT_MSG_BN_WHISPER_INFORM"
				)
			end
		end
		
		StaticPopupDialogs["SESSIONS_COPYURL"] = {
			text = "Ctrl-C to copy the link, then paste into your browser",
			button1 = DONE,
			hasEditBox = true,
			editBoxWidth = 260,
			enterClicksFirstButton = true,
			hideOnEscape = true,
			whileDead = true,
			OnShow = function(self, url) self.editBox:SetText(url) end,
			OnHide = function(self) self.data = nil end,
			EditBoxOnTextChanged = function(self, url) self:SetText(url); self:HighlightText() end,
			EditBoxOnEscapePressed = HideParentPanel,
			EditBoxOnEnterPressed = HideParentPanel,
		}
		
		Core.LSM = LibStub("LibSharedMedia-3.0")
		Core.Sessions:UpdateChatFont()
		
		ranOnce = true
		
	elseif e == "ADDON_LOADED" then
		local a = ...
		if a == ADDON_NAME then	
			if type(SESMSG_History) ~= "table" then
				SESMSG_History = {}
			end
			
			if SESMSG_Profile == nil then
				SESMSG_Profile = Core.defaultConfig
			
			else
				for k, v in pairs(Core.defaultConfig) do
					if SESMSG_Profile[k] == nil then
						SESMSG_Profile[k] = v
					end
				end
				if SESMSG_Profile["Version"] ~= Core.VERSION then
					for k, v in pairs(SESMSG_Profile) do
						if Core.defaultConfig[k] == nil then
							SESMSG_Profile[k] = nil
						end
					end
					
					if SESMSG_Profile["HistoryExpiry"] < 30 then
						SESMSG_Profile["HistoryExpiry"] = 14
					end
					
					SESMSG_Profile["Version"] = Core.VERSION
				end
			end
			
			Core.ChatFont = CreateFont("SessionsMessengerChatFont")
			Core.ChatFont:CopyFontObject("GameFontNormal")
			
			hub_Initialize()
			Core.db = SESMSG_Profile
			
			Core.LoadPreferences()
			
			SlashCmdList["SESSIONS_TOGGLEUI"] = function(msg)
				UI:Toggle()
			end
				SLASH_SESSIONS_TOGGLEUI1 = "/sessions"
				SLASH_SESSIONS_TOGGLEUI2 = "/sm"
				
			
			SlashCmdList["SESSIONS_CHECKVER"] = function(msg)
				Core.utils.SMPrint("Current version: ".. Core.VERSION)
			end			
				SLASH_SESSIONS_CHECKVER1 = "/smversion"
			
			
			BINDING_HEADER_SESSIONSMSGR = "Sessions Messenger"
			BINDING_NAME_SMToggle = "Show/Hide UI Toggle"
			BINDING_NAME_SMTalkToUnit = "Talk To Target (or mouseover)"
			BINDING_NAME_SMChatOverride = "Open Chat"

			self:UnregisterEvent("ADDON_LOADED")
		end
	
	elseif e == "PLAYER_LEAVING_WORLD" then
		if Core.db.LastHistoryCleanUp ~= date("%Y-%m-%d") then
			Core.History:CleanUp()
		end

	elseif e == "FRIENDLIST_UPDATE" then
		checkIngameFriends()
		self:UnregisterEvent("FRIENDLIST_UPDATE")
	end
end)

