local ADDON_NAME, Core = ...

--<
local date, time = date, time
local sub, gsub = string.sub, string.gsub
local upper, format, find, match = string.upper, string.format, string.find, string.match
local RGBToColorCode = RGBToColorCode
local UI = Core.UI
local utils = Core.utils
-->

local cache = {}

local PRESET_NEARBY = {"Nearby", "EMOTE", "SAY"}
local PRESET_PARTY =  {"Party", "PARTY"}
local PRESET_GUILD =  {"Guild", "GUILD"}

local Session = {
	savedText = "",
	missedMessages = 0,
	typeInform_lastSent = 0,
	typeInform_lastUpdated = 0,
	handshakeCD = 0,
	isDM = true,
	isMuted = false,
}

Session.__index = Session

local Messenger = {
	active = {},
	listeners = {},
	preset = {
		EMOTE = PRESET_NEARBY,
		SAY = PRESET_NEARBY,
		TEXT_EMOTE = PRESET_NEARBY,
		PARTY = PRESET_PARTY,
		PARTY_LEADER = PRESET_PARTY,
		GUILD = PRESET_GUILD
	}
}


-------------------------------------------------------------------
-------------------------------------------------------------------


local function SessionClass_new ()
	local class = table.remove(cache) or {}
	setmetatable(class, Session)
	
	return class
end


local function SessionClass_init (self, info)
	Messenger.active[self.name] = self
	
	self.page = Core.CreatePage(self)
	self.tab = Core.CreateTab(info, self)
	self.tab:SetScript("OnMouseDown", function(this, b)
		if b == "LeftButton" then
			Messenger:Switch(self)

		elseif b == "RightButton" then
			if IsLeftShiftKeyDown() then
				self:Deactivate()
			else
				self:ShowMenu()
			end
		end
	end)

	if Messenger.selected == nil then
		Messenger.selected = self
		self:SetSelected()
		UI:UpdateHeader(Messenger.selected)
		UI.editBox:SetTextColor(self:GetRGB())
		
	else
		self:Deselect()
	end
	
	if not self.isDM or self.needsInfo then return end
	
	if not UI:IsEventRegistered("CHAT_MSG_ADDON") then
		UI:RegisterEvent("CHAT_MSG_ADDON")
	end
	
	Core.History:OnSessionInit(self)
end


-------------------------------------------------------------------
-------------------------------------------------------------------


function Messenger.CreateFromGUID (guid)
	-- Creates a WHISPER session from GUID
	if not C_PlayerInfo.GUIDIsPlayer(guid) or Messenger:Get(guid) then
		return
	end
	
	local S = SessionClass_new()
	local info = {GetPlayerInfoByGUID(guid)}
	S:SetInfo(guid, unpack(info))
	
	SessionClass_init(S, info)
	table.insert(cache, wipe(info))
	return S
end


function Messenger.CreateFromID (id)
	-- Creates a BN_WHISPER session from bnetID
	if not BNConnected() then
		return utils.SMPrint("Could not create session: BattleNet not connected")
	elseif Messenger:Get(id) then
		return
	end
	
	local S = SessionClass_new()
	local info = C_BattleNet.GetAccountInfoByID(id)
	
	S.senderID = info.bnetAccountID
	S.title = info.accountName
	S.name = info.battleTag:upper()
	S.chatType = "BN_WHISPER"
	S.id = info.battleTag
	
	Messenger.listeners[id] = S.name
	SessionClass_init(S, info)
	return S
end


function Messenger.CreateFromCharacterName (name)
	-- Creates a WHISPER session without GUID. Has limited info: updates later if-when the GUID is encountered.
	if Messenger:Get(name) then
		return
	end
	
	local S = SessionClass_new()
	
	S.needsInfo = true
	S.senderID = name
	S.title = utils.capitalize(name)
	S.name = string.upper(name)
	S.chatType = "WHISPER"
	
	SessionClass_init(S)
end


function Messenger.CreateCustom (name, ...)
	-- Creates a session with 1+ chattypes. ex: CreateCustom("Name", "GUILD", "CHANNEL1", "CHANNEL3")
	if Messenger:Get(name) then
		return
	end
	
	local S = SessionClass_new()
	
	S.isDM = false
	S.title = name
	S.name = string.upper(name)	
	S.messengerIDs = {}
	
	for i=1, select("#", ...) do
		local ctype = select(i, ...)
		S.messengerIDs[i] = ctype
		Messenger.listeners[ctype] = S.name
		
		if ctype:match("PARTY") then
			Messenger.listeners["PARTY_LEADER"] = S.name
		elseif ctype:match("EMOTE") then
			Messenger.listeners["TEXT_EMOTE"] = S.name
		end
	end
	
	S.chatType = S.messengerIDs[1]
	
	if S.chatType:match("CHANNEL") then
		S.senderID = S.chatType:gsub("CHANNEL", "")
	else
		S.senderID = S.chatType
	end

	SessionClass_init(S)
	
	C_Timer.After(2, function()
		if Messenger:Get(S.name) and S.name == "NEARBY" then 
			S:UpdateTitle()
		end
	end)

	return S
end


function Messenger:Switch (to)
	if type(to) ~= "table" then
		to = self:Get(to)
		
		if not to then
			return
		end
	end
	
	if (self.selected ~= nil) and (self.selected ~= to) then
		self.selected:Deselect()
		self.selected.savedText = UI.editBox:GetText()
		
		self.selected = to
		to:SetSelected()
		to:StopGlowing()		
		
		tabTraverseIndex = to.tab.tabIndex
		UI:UpdateHeader(self.selected)
		UI.editBox:SetTextColor(self.selected:GetRGB())

	elseif (self.selected == to) then
		UI.editBox:SetFocus()
	end
end


function Messenger:GetSelectedRGB ()
	if self.selected then
		return self.selected:GetRGB()
		
	else
		return 1, 1, 1
	end
end


function Messenger:Get (tag)
	if type(tag) == "string" and not tag:match("^Player") then
		tag = upper(tag)
	end
	
	return self.active[tag] or self.active[self.listeners[tag]] or false
end


function Messenger:GetRoleplayAlias (name)
	if not Core.RPAddOn then return end
	name = utils.getFullName(name)
	local alias, customColour
	
	if Core.RPAddOn == "XRP" then
		alias = AddOn_XRP.Characters.byName[name].NA
		
	elseif Core.RPAddOn == "TRP3" then
		local profile
		
		profile = TRP3_API.utils.getCharacterInfoTab(name)
		
		if profile == nil or not next(profile) then
			return nil
		end
		
		alias = TRP3_API.chat.getFullnameUsingChatMethod(profile)
		customColour = profile.characteristics and profile.characteristics.CH or nil
		if customColour then customColour = "|cFF".. customColour end
	
	elseif Core.RPAddOn == "MRP" and msp.char[name] then
		alias = msp.char[name].field.NA
	end
	
	if (not alias or alias == "") then
		return
	else
		alias = alias:gsub('"', "'")
		if Core.db.StripColours then
			alias = alias:gsub("|[Cc]........", ""):gsub("|r", "")
		elseif customColour then
			alias = format("%s%s|r", customColour, alias)
		end
		return alias
	end
end


function Messenger:UpdateIfNeeded (name, guid)
	local S = self:Get(name)
	if not S then return end
	
	if S.needsInfo then
		S:SetInfo(guid, GetPlayerInfoByGUID(guid))
	end
end


function Messenger:CloseIfActive (tag)
	local session = self:Get(tag)
	
	if session then
		session:Deactivate()
	end
end


function Messenger:SetPreferences ()
	if not self.preferences then self.preferences = {} end

	for ctype, prefs in pairs(Core.db.ChatPreferences) do
		self.preferences[ctype] = self.preferences[ctype] and wipe(self.preferences[ctype]) or {}
		
		for setting, value in pairs(prefs) do
			self.preferences[ctype][setting] = value
		end
	end
	
	self.preferences["PARTY_LEADER"] = self.preferences["PARTY"]
	self.preferences["BN_WHISPER"] = self.preferences["WHISPER"]
	self.preferences["BN_WHISPER_INFORM"] = self.preferences["WHISPER"]
	self.preferences["WHISPER_INFORM"] = self.preferences["WHISPER"]
	self.preferences["TEXT_EMOTE"] = self.preferences["EMOTE"]
	
	if Core.ChatFont and Core.LSM then
		local path, size, flags = Core.ChatFont:GetFont()
		if (Core.LSM:Fetch("font", Core.db.TextFont) ~= path) or (Core.db.TextSize ~= size) or (Core.db.TextOutline ~= flags) then
			self:UpdateChatFont()
		end
	end
	
	local changeToVerbose = Core.db.UseVerbose
	for ctype, pattern in pairs(Core.formatPatterns.verbose) do
		Core.formatPatterns[ctype] = (changeToVerbose and pattern) or nil
	end
end


function Messenger:SendWho (who)
	C_FriendList.SetWhoToUi(true)
	FriendsFrame:UnregisterEvent("WHO_LIST_UPDATE")
	C_FriendList.SendWho(who)
end


function Messenger:OnWhoReceived ()
	for i=1, C_FriendList.GetNumWhoResults() do
		local info = C_FriendList.GetWhoInfo(i)
		local S = self:Get(utils.getFullName(info.fullName))

		if S then
			S:UpdateLevel(info.level)
		end
	end
	
	C_FriendList.SetWhoToUi(false)
	FriendsFrame:RegisterEvent("WHO_LIST_UPDATE")
end


function Messenger:UpdateChatFont ()
	local media = Core.LSM
	
	Core.ChatFont:SetFont(
		media:Fetch("font", Core.db.TextFont) or media:Fetch("font", media:GetDefault("font")),
		Core.db.TextSize,
		Core.db.TextOutline or nil
	)
	
	Core.ChatFont:SetShadowOffset(1, -1)
	
	for n, session in pairs(self.active) do
		session.page:SetFontObject(Core.ChatFont)
	end
end


-------------------------------------------------------------------
-------------------------------------------------------------------


local numPool = {}
for i=1, 9 do
	numPool[i] = tostring(i)
end
numPool[10] = "9+"


function Session:SetInfo (guid, ...)
	local _, class, _, race, gender, name, realm = ...
	if not guid or not name then return end
	
	self.senderID = utils.getFullName(name, realm)
	self.title = realm ~= "" and ( string.format("%s (%s)", name, realm) ) or name
	self.name = string.upper(self.senderID)
	self.chatType = "WHISPER"
	self.id = guid
	self.needsInfo = nil

	Messenger.listeners[guid] = self.name	
	
	if self.tab then
		gender = (gender == 2 and "Male") or (gender == 3 and "Female")
		self.tab.portrait:SetTexture("Interface/CHARACTERFRAME/TemporaryPortrait-"..gender.."-"..race)
		self.tab.sessionName:SetText(name)
		self.tab.sessionName:SetTextColor(GetClassColor(class))
		self.tab.infoString:SetText(utils.capitalize(class))
		
		Core.History:OnSessionInit(self)
	end
end

function Session:GetRGB (override)
	local c = ChatTypeInfo[override or self.chatType]
	return c.r, c.g, c.b

end

function Session:GetName ()
	return self.tab.sessionName:GetText()
end

function Session:GetID ()
	return self.senderID
end

function Session:Handshake ()
	if not(time() > self.handshakeCD) then
		return
	end

	ChatThrottleLib:SendAddonMessage("NORMAL", "SessionsMSGR", "SMCHK", "WHISPER", self.senderID)
	self.handshakeCD = time() + 5
	C_Timer.After(4, function() if self.isUsingSM == nil then self.isUsingSM = false end end)
end

function Session:UpdatePortrait (unit)
	SetPortraitTexture(self.tab.portrait, unit)
end

function Session:UpdateLevel (lvl)
	local infoString = self.tab.infoString:GetText()
	local startPoint, endPoint = infoString:find("%d+")

	if endPoint then
		infoString = infoString:sub(endPoint+2, #infoString)
	end
	
	self.tab.infoString:SetText(tostring(lvl).. " ".. infoString)
end

function Session:UpdateTitle (ctype)
	self.title = (ctype or string.format("%s (%s)", utils.capitalize(self.chatType), GetMinimapZoneText()))

	if Messenger.selected == self then
		UI:UpdateHeader(self)
	end
end

function Session:StartGlowing ()
	if not self.isMuted then
		self.tab.animation:Play()
	end
end

function Session:StopGlowing ()
	self.tab.animation:Stop()
end

function Session:SetSelected ()
	self.tab:SetBackdropBorderColor(1, 1, 1, 1)
	self.page:Show()
	self.page.slider:SetController(self.page)
	self:MarkAsRead()
	
	UI.editBox:SetText(self.savedText)
	UI.editBox:HighlightText()
	UI.editBox:SetFocus()
	
	UI.header.selectiveBtn:SetShown(not self.isDM)
	UI.header.selectiveBtn:SetChecked(self.selectiveModeEnabled)
end

function Session:IsSelected ()
	return Messenger.selected == self
end

function Session:MarkAsRead ()
	self.missedMessages = 0
	self.tab.notification:Hide()
	
	if self.notice then
		self.notice = Core.hub:RemoveNotice(self.notice)
	end	
end

function Session:Deselect ()
	self.tab:SetBackdropBorderColor(0, 0, 0, 1)
	self.page:Hide()
end

function Session:SetTyping ()
	if not(time() >= self.typeInform_lastUpdated) and not self.page:IsShown() then
		return
	end

	self.page:SetTypingStatus(self:GetName())
	self.typeInform_lastUpdated = time()+5

	C_Timer.After(6, function() if self.page then self.page:ClearTypingStatus() end end)
end

function Session:ShowMenu ()
	if self.menu == nil then
		local name = self:GetName()
		local hide = not self.isDM or self.needsInfo or false
		
		if not hide then
			name = format("%s%s|r", RGBToColorCode(self.tab.sessionName:GetTextColor()), name)
		end

		self.menu = {
			{text=name, isTitle=true, notCheckable=true},
			{
				text="Update Info",
				notCheckable=true,
				disabled=hide,
				func=function() Messenger:SendWho(self:GetName()) end
			},
			{
				text="View History",
				notCheckable=true,
				disabled=hide,
				func=function() Core.History:Fetch(self.name) end
			},
			{
				text="Add Friend",
				notCheckable=true,
				disabled=hide,
				func=function() C_FriendList.AddFriend(self.name) end
			},
			{
				text="Invite To Group",
				notCheckable=true,
				disabled=hide,
				func=function() C_PartyInfo.InviteUnit(self:GetName()) end
			},
			{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
			{
				text="Ignore",
				isNotRadio=true,
				disabled=hide,
				keepShownOnClick=true,
				checked=function() if not hide then return C_FriendList.IsIgnoredByGuid(self.id or UnitGUID("Player")) end end,
				func=function() C_FriendList.AddOrDelIgnore(self.name) end,
			},			
			{
				text="Mute Notifications",
				isNotRadio=true,
				checked=function() return self.isMuted end,
				func=function() self.isMuted = not self.isMuted; self:MarkAsRead() end,
			},
			{text="Close Session", notCheckable=true, func=function() self:Deactivate() end},
			{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
			{text="Cancel", notCheckable=true},
		}
	end	
	
	EasyMenu(self.menu, UI.floatingMenu, "cursor", 0, 0, "MENU")
	
end

function Session:UpdateNotice (ctype)
	self.missedMessages = self.missedMessages+1
	local numString = numPool[self.missedMessages] or numPool[10]
	
	if not self.tab.notification:IsShown() then
		self.tab.notification:Show()
	end
	self.tab.notification.num:SetText(numString)
	
	if not self.notice then
		if #Core.hub.notices < 3 then
			self.notice = Core.hub:AddNotice(self, ctype)
		else return
		end
	end

	self.notice.text:SetText(numString)
end

function Session:AddHistory (inbound, isBnet, ts, msg)
	if not Core.History:HasHistorySlot(self.name) then Core.History:NewEntry(self.name, self.id) end
	
	local link = SESMSG_History[self.name]
	local dt = date("%Y-%m-%d")
	
	if not link.INFO.MEM then
		local author = (inbound and Ambiguate(self:GetName(), "short")) or (Core.myCharacterName)
		
		link.INFO.MEM = {
			["MSG"] = author.. ": ".. msg,
			["DT"] = dt,
			["ON"] = Core.myCharacterName
		}
	end	
	
	if link.HISTORY[dt] == nil then
		link.HISTORY[dt] = {}
	end
	
	local slot = link.HISTORY[dt]
	
	slot[#slot+1] = {
		IN = inbound or (isBnet and false or Core.myCharacterName),
		TS = ts:gsub(" ", ""),
		MSG = msg
	}

end

function Session:Deactivate ()
	self:Deselect()
	Core.CloseTab(self.tab)
	self.page:Close()
	
	local DMTabsRemaining = false
	
	if #Core.orderedTabs == 0 then
		Messenger.selected = nil
		UI.editBox:SetText("")
		UI.editBox:SetTextColor(.8, .8, .8)
		UI:UpdateHeader(nil)
		UI.header.selectiveBtn:Hide()
		
	else
		for N, S in pairs(Messenger.active) do
			if S.tab.tabIndex == 1 then
				Messenger:Switch(S)
			end
			
			if S.isUsingSM == true then
				DMTabsRemaining = true
			end			
		end
	end
	
	for ctype, pointer in pairs(Messenger.listeners) do
		if pointer == self.name then
			Messenger.listeners[ctype] = nil
		end
	end

	Messenger.active[self.name] = nil
	if Messenger.lastWhisperTarget == self then Messenger.lastWhisperTarget = nil end
	if not DMTabsRemaining then UI:UnregisterEvent("CHAT_MSG_ADDON") end
	self:MarkAsRead()
	table.insert(cache, wipe(self))
end

Core.Sessions = Messenger
